Package: methyLImp2
Title: Missing value estimation of DNA methylation data
Version: 1.9.0
Authors@R: c(
    person("Pietro", "Di Lena", 
    role = c("aut"),
    comment = c(ORCID = "0000-0002-1838-8918")),
    person("Anna", "Plaksienko", 
    email = "anna@plaxienko.com", 
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-9607-057X")),
    person("Claudia", "Angelini", 
    role = c("aut"),
    comment = c(ORCID = "0000-0001-8350-8464")),
    person("Christine", "Nardini", 
    role = c("aut"),
    comment = c(ORCID = "0000-0001-7601-321X")))  
Description: This package allows to estimate missing values in DNA methylation data. methyLImp method is based on linear regression since methylation levels show a high degree of inter-sample correlation. Implementation is parallelised over chromosomes since probes on different chromosomes are usually independent. Mini-batch approach to reduce the runtime in case of large number of samples is available.
Imports: BiocParallel, parallel, stats, methods, corpcor,
        SummarizedExperiment, utils
Depends: R (>= 4.3.0), ChAMPdata
URL: https://github.com/annaplaksienko/methyLImp2
BugReports: https://github.com/annaplaksienko/methyLImp2/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
biocViews: DNAMethylation, Microarray, Software, MethylationArray,
        Regression
Suggests: BiocStyle, knitr, rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Language: en-US
git_url: https://git.bioconductor.org/packages/methyLImp2
git_branch: devel
git_last_commit: d5bec10
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:43:03 UTC; biocbuild
Author: Pietro Di Lena [aut] (ORCID: <https://orcid.org/0000-0002-1838-8918>),
  Anna Plaksienko [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9607-057X>),
  Claudia Angelini [aut] (ORCID: <https://orcid.org/0000-0001-8350-8464>),
  Christine Nardini [aut] (ORCID:
    <https://orcid.org/0000-0001-7601-321X>)
Maintainer: Anna Plaksienko <anna@plaxienko.com>
